@@ -13,7 +13,7 @@ from serverauditor_sshconfig.core.api import API
 from serverauditor_sshconfig.core.cryptor import RNCryptor
 from serverauditor_sshconfig.core.logger import PrettyLogger
 from serverauditor_sshconfig.core.ssh_config import SSHConfig
-from serverauditor_sshconfig.core.utils import parallel_map, p_input
+from serverauditor_sshconfig.core.utils import p_input, p_map
 
 
 class ExportSSHConfigApplication(SSHConfigApplication):
@@ -73,6 +73,7 @@ class ExportSSHConfigApplication(SSHConfigApplication):
 
         return
 
+    @description(valediction="OK!")
     def _choose_new_hosts(self):
         def get_prompt():
             if self._local_hosts:
@@ -113,7 +114,6 @@ class ExportSSHConfigApplication(SSHConfigApplication):
 
             self._logger.log(get_hosts_names(), color='blue')
 
-        self._logger.log("Ok!", color='green')
         if not self._local_hosts:
             self._valediction()
             sys.exit(0)
@@ -121,17 +121,41 @@ class ExportSSHConfigApplication(SSHConfigApplication):
 
     @description("Getting full information...")
     def _get_full_hosts(self):
+        def check_duplicates(hosts):
+            new_hosts = []
+            new_hosts_ids = set()
+            new_hosts_names = {}
+
+            # current serverauditor connections
+            for conn in self._sa_connections:
+                conn_id = self._get_sa_connection_uri(conn)
+                new_hosts_ids.add(conn_id)
+                new_hosts_names[conn_id] = self._get_sa_connection_name(conn)
+
+            for host in hosts:
+                host_id = '{host[user]}@{host[hostname]}:{host[port]}'.format(host=host)
+                if not host_id in new_hosts_ids:
+                    new_hosts_ids.add(host_id)
+                    new_hosts.append(host)
+                    new_hosts_names[host_id] = host['host']
+                else:
+                    self._logger.log('Seems "{cur_host}" is duplicate of "{ex_host}"!'.format(
+                        cur_host=host['host'],
+                        ex_host=new_hosts_names[host_id]
+                    ), color='blue')
+            return new_hosts
+
         def encrypt_host(host):
-            host['host'] = self._cryptor.encrypt(host['host'], self._sa_master_password)
-            host['hostname'] = self._cryptor.encrypt(host['hostname'], self._sa_master_password)
-            host['user'] = self._cryptor.encrypt(host['user'], self._sa_master_password)
+            host['host'] = self._cryptor.encrypt(host['host'])
+            host['hostname'] = self._cryptor.encrypt(host['hostname'])
+            host['user'] = self._cryptor.encrypt(host['user'])
             host['password'] = ''
 
             host['ssh_key'] = []
             for i, f in enumerate(host.get('identityfile', [])):
                 ssh_key = {
-                    'label': self._cryptor.encrypt(f[0], self._sa_master_password),
-                    'private_key': self._cryptor.encrypt(f[1], self._sa_master_password),
+                    'label': self._cryptor.encrypt(f[0]),
+                    'private_key': self._cryptor.encrypt(f[1]),
                     'public_key': '',
                     'passphrase': ''
                 }
@@ -139,7 +163,8 @@ class ExportSSHConfigApplication(SSHConfigApplication):
             return host
 
         almost_full_local_hosts = [self._config.get_host(h, substitute=True) for h in self._local_hosts]
-        self._full_local_hosts = parallel_map(encrypt_host, almost_full_local_hosts)
+        full_local_hosts = check_duplicates(almost_full_local_hosts)
+        self._full_local_hosts = p_map(encrypt_host, full_local_hosts)
         return
 
     @description("Creating keys and connections...")
@@ -156,6 +181,7 @@ def main():
         sys.exit(1)
     return
 
+
 if __name__ == "__main__":
 
     main()
\ No newline at end of file
